﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestLineChart
{
    public class ConnectLinker : ContentControl
    {
        internal double TranslateTolorrence
        {
            get { return (double)GetValue(TranslateTolorrenceProperty); }
            set { SetValue(TranslateTolorrenceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for TranslateTolorrence.  This enables animation, styling, binding, etc...
        internal static readonly DependencyProperty TranslateTolorrenceProperty =
            DependencyProperty.Register("TranslateTolorrence", typeof(double), typeof(ConnectLinker), new UIPropertyMetadata(0.0d));


        public static ConnectLinker GetLinker(DependencyObject obj)
        {
            return (ConnectLinker)obj.GetValue(LinkerProperty);
        }

        public static void SetLinker(DependencyObject obj, ConnectLinker value)
        {
            obj.SetValue(LinkerProperty, value);
        }

        // Using a DependencyProperty as the backing store for Linker.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty LinkerProperty =
            DependencyProperty.RegisterAttached("Linker", typeof(ConnectLinker), typeof(ConnectLinker), new UIPropertyMetadata(null, new PropertyChangedCallback(LinkChangedCallBack)));

        
        private List<UIElement> Points { get; set; }

        public ConnectLinker()
        {
            Points = new List<UIElement>();
        }

        private static void LinkChangedCallBack(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            ConnectLinker linker = e.NewValue as ConnectLinker;
            linker.Points.Add(sender as UIElement);
            linker.InvalidateVisual();
        }

        protected override void OnRender(DrawingContext drawingContext)
        {
            PathGeometry geometry = new PathGeometry();
            Point? previousPoint = null;
            int index = 0;
            Brush brush = null;
            List<Point> debugPoints = new List<Point>();
            for (int i = 0; i <= Points.Count - 1; ++i)
            {
                Rectangle e = Points[i] as Rectangle;
                if (brush == null)
                {
                    brush = e.Fill;
                }

                if (previousPoint == null)
                {
                    previousPoint = new Point(Canvas.GetLeft(e), Canvas.GetBottom(e));
                }
                else
                {
                    PathFigure figure = new PathFigure();

                    LineSegment segment = new LineSegment();
                    Canvas parent = LogicalTreeHelper.GetParent(e) as Canvas;
                    Point point = new Point(parent.ActualWidth * index, Canvas.GetBottom(e));
                    segment.Point = point;
                    figure.StartPoint = previousPoint.Value;

                    PathSegmentCollection segments = new PathSegmentCollection();
                    segments.Add(segment);
                    figure.Segments = segments;

                    previousPoint = point;
                    geometry.Figures.Add(figure);
                }

                debugPoints.Add(previousPoint.Value);
                index++;
            }

            if (brush != null)
            {
                drawingContext.PushTransform(new ScaleTransform(1, -1, 0, ActualHeight / 2 - TranslateTolorrence / 2));
                drawingContext.PushTransform(new TranslateTransform(3, 0));
                drawingContext.DrawGeometry(brush, new Pen(brush, 2), geometry);
            }
        }
    }
}
