﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Diagnostics;
using System.Collections.ObjectModel;
using System.IO;
using System.Threading;

namespace TestLineChart
{
    /// <summary>
    /// Interaction logic for LineChart.xaml
    /// </summary>
    public partial class LineChart : UserControl
    {
        public double Maximum
        {
            get { return (double)GetValue(MaximumProperty); }
            set { SetValue(MaximumProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Maximum.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MaximumProperty =
            DependencyProperty.Register("Maximum", typeof(double), typeof(LineChart), new UIPropertyMetadata(0.0d));

        public double Minimum
        {
            get { return (double)GetValue(MinimumProperty); }
            set { SetValue(MinimumProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Minimum.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MinimumProperty =
            DependencyProperty.Register("Minimum", typeof(double), typeof(LineChart), new UIPropertyMetadata(0.0d));

        public double TickLabelHeight
        {
            get { return (double)GetValue(TickLabelHeightProperty); }
            set { SetValue(TickLabelHeightProperty, value); }
        }

        // Using a DependencyProperty as the backing store for TickLabelHeight.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TickLabelHeightProperty =
            DependencyProperty.Register("TickLabelHeight", typeof(double), typeof(LineChart), new UIPropertyMetadata(30.0d));

        private Brush[] m_Brushes = new Brush[] { Brushes.Blue, Brushes.Red, Brushes.Green, Brushes.Yellow, Brushes.Orange,
            Brushes.Purple};
        private int m_BrushIndex = 0;

        public LineChart()
        {
            InitializeComponent();

            ObjectDataProvider provider = FindResource("LineChartReference") as ObjectDataProvider;
            provider.ObjectInstance = this;
        }

        internal class HistogramDataObject
        {
            public object Category { get; set; }

            public double Value { get; set; }

            public Brush Fill { get; set; }
        }

        internal class ChartLine
        {
            public string SeriesName { get; set; }

            public HistogramValueCollection Values { get; set; }

            public Brush Fill { get; set; }
        }

        public void AddChartData(Array array)
        {
            //List<ChartLine> data = null;
            HistogramValueCollection values = null;

            //if (DataContext == null)
            //{
            //    data = new List<ChartLine>();
            //}
            //else
            //{
            //    data = DataContext as List<ChartLine>;
            //}

            //Debug.Assert(data != null);
            values = PopulateArray(this, array);

            double maximum = values.Max(v => v.Value);
            Maximum = Math.Max(maximum, Maximum);
            double minimum = values.Min(v => v.Value);
            minimum = Math.Min(minimum, Minimum);

            foreach (HistogramDataObject h in values)
            {
                h.Fill = m_Brushes[m_BrushIndex];
            }

            //data.Add(new ChartLine()
            //{
            //    Values = values,
            //    Fill = m_Brushes[m_BrushIndex]
            //});
            m_BrushIndex++;

            DataContext = values;
        }

        private static HistogramValueCollection PopulateArray(LineChart chart, Array array)
        {
            Trace.WriteLineIf(array.Rank > 1, "Warning, multiple dimentional array is treated as 1-D array");
            HistogramValueCollection histogramDataObjects = new HistogramValueCollection();
            int index = 0;
            foreach (object a in array)
            {
                if (!(a is IConvertible))
                    throw new InvalidDataException("Only arrays which host IComparable object are supported");

                histogramDataObjects.Add(new HistogramDataObject() { Category = index++, Value = ((IConvertible)a).ToDouble(Thread.CurrentThread.CurrentUICulture) });
            }

            return histogramDataObjects;
        }

    }

    internal class HistogramValueCollection : ObservableCollection<LineChart.HistogramDataObject>
    {
    }
}
